==> nhl97-ts_utils-1.6.2.tgz: package/src/TypeUtils.ts <==
export type Fn<A, B> = (p: A) => B

export type AmountWithUnitA<A> = {
  amount: number;
  unit: A;
}

// transforms the structured type to how you'd write it in CSS
export function to_CSS_string(len: AmountWithUnitA<CSSLengthUnit>) {
  return len.amount.toString() + len.unit
}

/**
 * copied from the Typescript Handbook
 * IDK about the Arr type... e.g. why not 'any[]'
 */
export function partialCall<T extends Arr, U extends Arr, R>(
  f: (...args: [...T, ...U]) => R,
  ...headArgs: T
) {
  return (...tailArgs: U) => f(...headArgs, ...tailArgs);
}
type Arr = readonly unknown[]

/**
 * TYPE/CLASS BUILDING BLOCKS
 */
export type IdAble<A> = { id: A }
export type LimitAble<A> = { limit: A }
export type KindAble = { kind: string }

/**
 * after some reading and deliberation, I've come to prefer string literal types (~= union types) over enums
 * [best comparison article I've found](https://blog.bam.tech/developer-news/should-you-use-enums-or-union-types-in-typescript)
 * To get both the relevant type ("x is one of the enumerated (string) values"),
 * and a way (specifically, an array) to iterate over the values, use the following pattern.
 * notice the 'as const', the different capitalizations, and the trailing 's' on the array
 * I don't know why the '[number]' thing is needed/works
 */
const someSemanticValues = ['FirstValue', 'SecondOne', 'AndSoOn'] as const;
type SomeSemanticValue = (typeof someSemanticValues)[number];


export type Dir4 = (typeof dir4s)[number];
export const dir4s = ["North", "East", "South", "West"] as const;


/**
 * Type: DIRection KeyCodes
 * (small subset of KeyboardEvent.key values)
 */
export type DirKC = (typeof dirKCs)[number];
export const dirKCs = ["ArrowUp", "ArrowRight", "ArrowDown", "ArrowLeft"] as const
export type UDKC = (typeof udKCs)[number]
export const udKCs = ["ArrowDown", "ArrowUp"] as const
export type SpaceOrEnterKC = (typeof spaceOrEnterKCs)[number]
export const spaceOrEnterKCs = [" ", "Enter"] as const

// CSS VALUES
export type SizingInfoList =
  BoxSize     // set+limit to specific dimensions
  | ElemLimit // set size according to an amount of items that should be fully visible at a time
  | { kind: "Stretch" } // size like parent
  | { kind: "Grow" };   // don't limit/set size, let ol grow to hold all items

// set+limit to specific dimensions
export type BoxSize = {
  kind: "BoxSize"
  height: AmountWithUnitA<CSSLengthUnit>;
  width: AmountWithUnitA<CSSLengthUnit>
}
export type ElemLimit = LimitAble<BigInt> & { kind: "ElemLimit" }

export type SizingStratList = "BoxSize" | "ElemLimit" | "Stretch" | "Grow"

export type PositioningStrat = (typeof positioningStrats)[number];
export const positioningStrats = ["start", "center", "end", "space-between"] as const;
// CSS length units (only the subset I think is useful/needed)
export type CSSLengthUnit = (typeof cssLengthUnits)[number];
export const cssLengthUnits = ["px", "rem", "em", "vh", "vw"] as const;


// TYPE GUARDS

/**
 *
 * @param value to assert type over
 * @param candidates the possible (string-y) values of the type
 * @returns
 */
export function isStringLiteralOf<T extends string>(value: any, candidates: readonly T[]): value is T {
  return candidates.includes(value);
}

export function isFreeOfNulls<T>(array: T[]): array is NonNullable<T>[] {
  return array.every((elem) => elem !== null);
}

// ENUM UTILITIES

/**
 * copied from somewhere, don't remember, don't know if/how it works
 *
 * @param enum_type an enum _type_ (so I assume 'typeof enumObj'?)
 *
 * @returns an array of the enums values
 */
export function enumKeys<O extends object, K extends keyof O = keyof O>(enum_type: O): K[] {
  return Object.keys(enum_type).filter(k => Number.isNaN(+k)) as K[];
}

==> nhl97-ts_utils-1.6.3.tgz: package/src/TypeUtils.ts <==
export type Fn<A, B> = (p: A) => B

export type AmountWithUnitA<A> = {
  amount: number;
  unit: A;
}

export type CSSLengthSpec = AmountWithUnitA<CSSLengthUnit>

// transforms the structured type to how you'd write it in CSS
export function to_CSS_string(len: CSSLengthSpec) {
  return len.amount.toString() + len.unit
}

/**
 * copied from the Typescript Handbook
 * IDK about the Arr type... e.g. why not 'any[]'
 */
export function partialCall<T extends Arr, U extends Arr, R>(
  f: (...args: [...T, ...U]) => R,
  ...headArgs: T
) {
  return (...tailArgs: U) => f(...headArgs, ...tailArgs);
}
type Arr = readonly unknown[]

/**
 * TYPE/CLASS BUILDING BLOCKS
 */
export type IdAble<A> = { id: A }
export type LimitAble<A> = { limit: A }
export type KindAble = { kind: string }

/**
 * after some reading and deliberation, I've come to prefer string literal types (~= union types) over enums
 * [best comparison article I've found](https://blog.bam.tech/developer-news/should-you-use-enums-or-union-types-in-typescript)
 * To get both the relevant type ("x is one of the enumerated (string) values"),
 * and a way (specifically, an array) to iterate over the values, use the following pattern.
 * notice the 'as const', the different capitalizations, and the trailing 's' on the array
 * I don't know why the '[number]' thing is needed/works
 */
const someSemanticValues = ['FirstValue', 'SecondOne', 'AndSoOn'] as const;
type SomeSemanticValue = (typeof someSemanticValues)[number];


export type Dir4 = (typeof dir4s)[number];
export const dir4s = ["North", "East", "South", "West"] as const;


/**
 * Type: DIRection KeyCodes
 * (small subset of KeyboardEvent.key values)
 */
export type DirKC = (typeof dirKCs)[number];
export const dirKCs = ["ArrowUp", "ArrowRight", "ArrowDown", "ArrowLeft"] as const
export type UDKC = (typeof udKCs)[number]
export const udKCs = ["ArrowDown", "ArrowUp"] as const
export type SpaceOrEnterKC = (typeof spaceOrEnterKCs)[number]
export const spaceOrEnterKCs = [" ", "Enter"] as const

// CSS VALUES
export type SizingInfoList =
  BoxSize     // set+limit to specific dimensions
  | ElemLimit // set size according to an amount of items that should be fully visible at a time
  | { kind: "Stretch" } // size like parent
  | { kind: "Grow" };   // don't limit/set size, let ol grow to hold all items

// set+limit to specific dimensions
export type BoxSize = {
  kind: "BoxSize"
  height: CSSLengthSpec;
  width: CSSLengthSpec
}
export type ElemLimit = LimitAble<BigInt> & { kind: "ElemLimit" }

export type SizingStratList = "BoxSize" | "ElemLimit" | "Stretch" | "Grow"

export type PositioningStrat = (typeof positioningStrats)[number];
export const positioningStrats = ["start", "center", "end", "space-between"] as const;
// CSS length units (only the subset I think is useful/needed)
export type CSSLengthUnit = (typeof cssLengthUnits)[number];
export const cssLengthUnits = ["px", "rem", "em", "vh", "vw"] as const;


// TYPE GUARDS

/**
 *
 * @param value to assert type over
 * @param candidates the possible (string-y) values of the type
 * @returns
 */
export function isStringLiteralOf<T extends string>(value: any, candidates: readonly T[]): value is T {
  return candidates.includes(value);
}

export function isFreeOfNulls<T>(array: T[]): array is NonNullable<T>[] {
  return array.every((elem) => elem !== null);
}

// ENUM UTILITIES

/**
 * copied from somewhere, don't remember, don't know if/how it works
 *
 * @param enum_type an enum _type_ (so I assume 'typeof enumObj'?)
 *
 * @returns an array of the enums values
 */
export function enumKeys<O extends object, K extends keyof O = keyof O>(enum_type: O): K[] {
  return Object.keys(enum_type).filter(k => Number.isNaN(+k)) as K[];
}
